import java.io.*;
import java.util.*;

public class TestGen {

    public static void main(String[] args) throws Exception {
        new File("../tests").mkdir();

        done = 0;
        genSamples();
        genSimples();

        genFaFa(10);
        genImpossible1(7);
        genPossible1(5);
        genPossible2(5);
        genPossible3(5);
        genMaybePossible1(5);
        genMaybePossible2(5);
        genMaybePossible3(5);

        for (int i = 300; i <= 3000; i += 300) {
            genMaybePossible1(near(i));
            genMaybePossible2(near(i));
            genMaybePossible3(near(i));
            genFaFa(near(i));
        }

        genFaFa(MAXX);
        genImpossible1(MAXN);
        genPossible1(MAXX);
        genPossible2(MAXX);
        genPossible3(MAXX);
        genMaybePossible1(MAXX);
        genMaybePossible2(MAXX);
        genMaybePossible3(MAXX);

    }

    static class Pair {
        int t, a;
        Pair(){}
        Pair(int x, int y) {
            this.t = x;
            this.a = y;
        }
    }

    static int done = 0;
    static final Random rnd = new Random(3289214075201L);
    static final int MAXN = 10_000;
    static final int MAXX = MAXN / 3;
    static final String vowels = "aoeuyi";


    static int rand(int n) {
        if (n < 1)
            return 0;
        return rnd.nextInt(n);
    }

    static int near(int n) {
        int x = rand(n / 4 + 1);
        return n - n / 4 + x;
    }

    static int nand(int n) {
        return rand(n / 2) + n / 2;
    }

    static boolean trueOrFalse() {
        return rnd.nextBoolean();
    }

    static void print(String s) throws IOException {
        done++;
        System.err.println("Generation test #" + done);
        PrintWriter out;
        if (done < 10)
            out = new PrintWriter("../tests/0" + String.valueOf(done));
        else
            out = new PrintWriter("../tests/" + String.valueOf(done));
        out.println(s);
        out.close();
    }

    //begin generators

    static void genSamples() throws IOException {
        print("hefallofa");
        print("abc");
    }

    static void genSimples() throws IOException {
        print("hefallofafa");
        print("fafa");
        print("fafafa");
        print("afafa");
        print("afafafa");
        print("abracadabra");
        print("afabrafacafadafabrafa");
        print("afabrafacafdafabrafa");
    }

    static char genChar() {
        return (char) ('a' + rand(26));
    }

    static char genVowel() {
        int i = rand(vowels.length());
        return vowels.charAt(i);
    }

    static String genRandString(int n) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < n; i++) {
            sb.append(genChar());
        }
        return sb.toString();
    }

    static String genVowelString(int n) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < n; i++) {
            sb.append(genVowel());
        }
        return sb.toString();
    }

    static String genManyVowelString(int n) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < n; i++) {
            if (trueOrFalse()) {
                sb.append(genChar());
            } else {
                sb.append(genVowel());
            }
        }
        return sb.toString();
    }

    static void genImpossible1(int n) throws IOException {
        int x = rand(3);
        String s;
        if (x == 0) {
            s = genRandString(n);
        } else if (x == 1) {
            s = genVowelString(n);
        } else {
            s = genManyVowelString(n);
        }
        print(s);
    }

    static boolean isVovel(char c) {
        boolean res = false;
        for (int j = 0; j < vowels.length(); j++) {
            res |= (vowels.charAt(j) == c);
        }
        return res;
    }

    static int countVowels(String s) {
        int ans = 0;
        for (int i = 0; i < s.length(); i++) {
            if (isVovel(s.charAt(i)))
                ans++;
        }
        return ans;
    }

    static void genPossible1(int n) throws IOException {
        String s = genRandString(n);
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < s.length(); i++) {
            sb.append(s.charAt(i));
            if (isVovel(s.charAt(i)))
                sb.append("fa");
        }
        print(sb.toString());
    }

    static void genPossible2(int n) throws IOException {
        String s = genVowelString(n);
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < s.length(); i++) {
            sb.append(s.charAt(i));
            if (isVovel(s.charAt(i)))
                sb.append("fa");
        }
        print(sb.toString());
    }
    static void genPossible3(int n) throws IOException {
        String s = genManyVowelString(n);
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < s.length(); i++) {
            sb.append(s.charAt(i));
            if (isVovel(s.charAt(i)))
                sb.append("fa");
        }
        print(sb.toString());
    }


    static void genFaFa(int n) throws IOException {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < n / 2 - 2; i++) {
                sb.append("fa");
        }
        print(sb.toString());
        sb.append("fa");
        print(sb.toString());
    }

    static void genMaybePossible1(int n) throws IOException {
        String s = genRandString(n);
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < s.length(); i++) {
            sb.append(s.charAt(i));
            if (isVovel(s.charAt(i)))
                sb.append("fa");
        }
        int q = rand(10) + 1;
        while (q-- > 0) {
            int i = rand(sb.length());
            sb.deleteCharAt(i);
        }
        print(sb.toString());
    }

    static void genMaybePossible2(int n) throws IOException {
        String s = genVowelString(n);
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < s.length(); i++) {
            sb.append(s.charAt(i));
            if (isVovel(s.charAt(i)))
                sb.append("fa");
        }
        int q = 1;
        while (q-- > 0) {
            int i = rand(sb.length());
            sb.deleteCharAt(i);
        }
        print(sb.toString());
    }
    static void genMaybePossible3(int n) throws IOException {
        String s = genManyVowelString(n);
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < s.length(); i++) {
            sb.append(s.charAt(i));
            if (isVovel(s.charAt(i)))
                sb.append("fa");
        }
        int q = rand(5) + 1;
        while (q-- > 0) {
            int i = rand(sb.length());
            sb.deleteCharAt(i);
        }
        print(sb.toString());
    }


    //end generator

}
