import java.util.*;
import java.io.*;

public class expression_dp_wa {
    FastScanner in;
    PrintWriter out;

    private static final long MOD = 1_000_000_007;
        
    private class ParserException extends Exception {
        public ParserException(String message) {
            super(message);
        }
    }
    
    private class Node {
        long value;
        int index;
        char sign;
        Node left;
        Node right;

        Node(long value) {
            this.value = value;
            this.index = -1;
            sign = '_';
            left = null;
            right = null;
        }
        
        Node(char c, int index, Node left, Node right) {
            this.sign = c;
            this.index = index;
            if (c == '+') {
                this.value = left.value + right.value;
            } else {
                this.value = left.value * right.value;
            }
            this.value %= MOD;
            this.left = left;
            this.right = right;
        }

        public void add(long x) {
            value += x;
            if (value >= MOD) {
                value -= MOD;
            }
            if (value < 0) {
                value += MOD;
            }
        }

        public void mul(long x) {
            value = ((value * x) % MOD + MOD) % MOD;
        }
    }

    private class ExpressionParser {
        private String expression;
        private int index;
        private long value;

        private boolean isDigit(char c) {
            return '0' <= c && c <= '9';
        }

        private void skipSpaces() {
            while (index < expression.length() && Character.isWhitespace(expression.charAt(index))) {
                index++;
            }
        }
        
        private void parseInt() {
            value = 0;
            while (index < expression.length() && isDigit(expression.charAt(index))) {
                value = (value * 10 + expression.charAt(index) - '0') % MOD;
                index++;
            }
        }
        
        private Node parseMultiplier() throws ParserException {
            skipSpaces();
            if (index >= expression.length()) {
                throw new ParserException("Unexpected end of expression");
            }
            if (isDigit(expression.charAt(index))) {
                parseInt();
                skipSpaces();

                return new Node(value);
            } else if (expression.charAt(index) == '(') {
                index++;
                Node result = parseExpression();
                if (index >= expression.length() || expression.charAt(index) != ')') {
                    throw new ParserException("No pair for open parenthesis");
                }
                index++;
                skipSpaces();
                return result;
            } 
            throw new ParserException("Wrong symbol at position " + (index + 1) + ": " + expression.charAt(index));
        }

        private Node parseSummand() throws ParserException {
            Node result = parseMultiplier();
            skipSpaces();

            while (index < expression.length() && expression.charAt(index) == '*') {
                index++;
                result = new Node('*', index - 1, result, parseMultiplier());
                skipSpaces();
            }
            return result;
        }

        private Node parseExpression() throws ParserException {
            Node result = parseSummand();
            skipSpaces();

            while (index < expression.length() && expression.charAt(index) == '+') {
                index++;
                result = new Node('+', index - 1, result, parseSummand());
                skipSpaces();
            }
            return result;
        }

        public Node parse(String expression) {
            this.expression = expression;
            value = 0;
            index = 0;
            try {
                Node root = parseExpression();
                if (index != expression.length()) {
                    throw new ParserException("Extra information at the end of expression");
                }
                return root;
            } catch (Throwable e) {
                throw new AssertionError(e.getMessage());
            }
        }
    }
    
    public void dfs(Node root, long[] answer, long a, long b) {
        if (root.left == null || root.right == null) {
            return;
        }
        assert (root.sign == '+' || root.sign == '*');
        if (root.sign == '+') {
            answer[root.index] = ((a * root.value + b) % MOD + MOD) % MOD;
            answer[root.index] = ((answer[root.index] - root.value) % MOD + MOD) % MOD;
            answer[root.index] = (answer[root.index] + root.left.value * root.right.value) % MOD;
            dfs(root.left, answer, a, (b + root.right.value) % MOD);
            dfs(root.right, answer, a, (b + root.left.value) % MOD);
        } else {
            answer[root.index] = ((a * root.value + b) % MOD + MOD) % MOD;
            answer[root.index] = ((answer[root.index] - root.value) % MOD + MOD) % MOD;
            answer[root.index] = (answer[root.index] + root.left.value + root.right.value) % MOD;
            dfs(root.left, answer, (a * root.right.value) % MOD, b);
            dfs(root.right, answer, (a * root.left.value) % MOD, b);
        }
    }

    public void solve() throws IOException {
        String expression = in.nextLine();
        int n = in.nextInt();
        ExpressionParser parser = new ExpressionParser();
        Node root = parser.parse(expression);
        
        long[] answer = new long[expression.length()];
        dfs(root, answer, 1L, 0L);
        
        for (int i = 0; i < n; i++) {
            int index = in.nextInt() - 1;
            out.println(answer[index]);
        }
    }

    public void run() {
        try {
            in = new FastScanner(new File("expression.in"));
            out = new PrintWriter(new File("expression.out"));

            solve();

            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    class FastScanner {
        BufferedReader br;
        StringTokenizer st;

        FastScanner(File f) {
            try {
                br = new BufferedReader(new FileReader(f));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }

        String nextLine() {
            try {
                return br.readLine();
            } catch (IOException e) {
                e.printStackTrace();
            }
            return null;
        }
        
        String next() {
            while (st == null || !st.hasMoreTokens()) {
                try {
                    String s = br.readLine();
                    st = new StringTokenizer(s);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            return st.nextToken();
        }

        int nextInt() {
            return Integer.parseInt(next());
        }
    }

    public static void main(String[] arg) {
        new expression_dp_wa().run();
    }
}
