#include <cstdio>
#include <vector>
#include <algorithm>
#include <set>
#include <string>
#include <iostream>
#include <cassert>

#pragma comment(linker, "/STACK:256000000")

using namespace std;

const int MAXN = 1 << 19;
const int MOD = 1000000007;

struct node {
    char operation;
    int value;
    vector<int> children;

    node() {}
};

string s;
int n;
int k, b[MAXN];
node t[MAXN];
int sz;
int ptr;
pair<int, int> pos[MAXN];
vector<int> ifChanged[MAXN];

int parseNumber() {
    int res = sz++;
    t[res].operation = 0;
    t[res].value = 0;
    while (ptr < n && s[ptr] >= '0' && s[ptr] <= '9') {
        t[res].value = (10LL * t[res].value + (s[ptr++] - '0')) % MOD;
    }
    return res;
}

int parseExpression();

int parseMultiplier() {
    assert(ptr < n && (s[ptr] >= '0' && s[ptr] <= '9' || s[ptr] == '('));
    if (s[ptr] == '(') {
        ptr++;
        int res = parseExpression();
        assert(ptr < n && s[ptr++] == ')');
        return res;
    }
    return parseNumber();
}

int parseSummand() {
    int res = sz++;
    t[res].operation = '*';
    t[res].children.push_back(parseMultiplier());
    t[res].value = (t[t[res].children.back()].value) % MOD;
    while (ptr < n && s[ptr] == '*') {
        pos[ptr++] = make_pair(res, t[res].children.size() - 1);
        t[res].children.push_back(parseMultiplier());
        t[res].value = (1LL * t[res].value * t[t[res].children.back()].value) % MOD;
    }
    assert(ptr == n || s[ptr] == ')' || s[ptr] == '+');
    return res;
}

int parseExpression() {
    int res = sz++;
    t[res].operation = '+';
    t[res].children.push_back(parseSummand());
    t[res].value = (t[t[res].children.back()].value) % MOD;
    while (ptr < n && s[ptr] == '+') {
        pos[ptr++] = make_pair(res, t[res].children.size() - 1);
        t[res].children.push_back(parseSummand());
        t[res].value = (t[res].value + t[t[res].children.back()].value) % MOD;
    }
    assert(ptr == n || s[ptr] == ')');
    return res;
}

void dfs(int v, int A, int B) { 
    vector<int> pref(t[v].children.size());
    vector<int> suf(t[v].children.size());

    if (t[v].operation == '+') {
        pref[0] = t[t[v].children[0]].value;
        for (int i = 1; i < (int)t[v].children.size(); i++) {
            pref[i] = (pref[i - 1] + t[t[v].children[i]].value) % MOD;
        }
        suf[(int)t[v].children.size() - 1] = t[t[v].children.back()].value;
        for (int i = (int)t[v].children.size() - 2; i >= 0; i--) {
            suf[i] = (suf[i + 1] + t[t[v].children[i]].value) % MOD;
        }
    }
    if (t[v].operation == '*') {
        pref[0] = t[t[v].children[0]].value;
        for (int i = 1; i < (int)t[v].children.size(); i++) {
            pref[i] = (1LL * pref[i - 1] * t[t[v].children[i]].value) % MOD;
        }
        suf[(int)t[v].children.size() - 1] = t[t[v].children.back()].value;
        for (int i = (int)t[v].children.size() - 2; i >= 0; i--) {
            suf[i] = (1LL * suf[i + 1] * t[t[v].children[i]].value) % MOD;
        }
    }

    for (int i = 0; i < (int)t[v].children.size(); i++) {
        int nA, nB;
        if (t[v].operation == '+') {
            nA = A;
            nB = B;
            if (i > 0) nB = (nB + 1LL * pref[i - 1] * A) % MOD;
            if (i + 1 < (int)t[v].children.size()) nB = (nB + 1LL * suf[i + 1] * A) % MOD;
        }
        if (t[v].operation == '*') {
            nA = A;
            nB = B;
            if (i > 0) nA = (1LL * nA * pref[i - 1]) % MOD;
            if (i + 1 < (int)t[v].children.size()) nA = (1LL * nA * suf[i + 1]) % MOD;
        }
        dfs(t[v].children[i], nA, nB);
    }

    for (int i = 0; i + 1 < (int)t[v].children.size(); i++) {
        int nvalue;
        if (t[v].operation == '+') {
            nvalue = (1LL * t[t[v].children[i]].value * t[t[v].children[i + 1]].value) % MOD;
            if (i > 0) nvalue = (nvalue + pref[i - 1]) % MOD;
            if (i + 2 < (int)t[v].children.size()) nvalue = (nvalue + suf[i + 2]) % MOD;
            ifChanged[v].push_back((1LL * nvalue * A + B) % MOD);
        }
        if (t[v].operation == '*') {
            nvalue = (pref[i] + suf[i + 1]) % MOD;
            ifChanged[v].push_back((1LL * nvalue * A + B) % MOD);
        }
    }
}

int main() {
    freopen("expression.in", "r", stdin);
    freopen("expression.out", "w", stdout);

    ios_base::sync_with_stdio(0);

    getline(cin, s);
    n = s.length();
    sz = 0;
    ptr = 0;
    int root = parseExpression();
    dfs(root, 1, 0);
    cin >> k;
    for (int i = 0; i < k; i++) {
        cin >> b[i];
        b[i]--;
        cout << ifChanged[pos[b[i]].first][pos[b[i]].second] << "\n";
    }

    return 0;
}