import java.io.*;
import java.util.*;

class Pair<F, S> {
    public F first;
    public S second;
    public Pair(F first, S second) {
        this.first = first;
        this.second = second;
    }
}

class Dsu {
    private int[] p;
    private int[] rank;

    public Dsu(int n) {
        p = new int[n];
        rank = new int[n];
        for (int i = 0; i < n; ++i) {
            p[i] = i;
        }
    }

    public int root(int v) {
        if (v == p[v]) {
            return v;
        }
        p[v] = root(p[v]);
        return p[v];
    }

    public boolean join(int x, int y) {
        x = root(x);
        y = root(y);
        if (x == y) {
            return false;
        }
        if (rank[x] < rank[y]) {
            int tmp = x;
            x = y;
            y = tmp;
        }
        p[y] = x;
        if (rank[x] == rank[y]) {
            ++rank[x];  
        }
        return true;
    }
}

class Tree {
    public ArrayList<ArrayList<Pair<Integer, Edge> > > g;
    private int degree[];
    private Edge[] edges;
    private int ptr = 0;

    static class Edge {
        public int from;
        public int to;
        public int l;
        public int t;
        public Edge edge;

        public Edge(int from, int to) {
            this.from = from;
            this.to = to;
        }

        public Edge(int from, int to, int l, int t) {
            this.from = from;
            this.to = to;
            this.l = l;
            this.t = t;
        }

        public void setEdge(Edge e) {
            edge = e;
        }
    }

    public Tree(int n) {
        edges = new Edge[n - 1];
        g = new ArrayList<ArrayList<Pair<Integer, Edge>>>();
        degree = new int[n];
        for (int i = 0; i < n; ++i)
            g.add(new ArrayList<Pair<Integer, Edge>>());
    }

    public Edge[] getEdges() {
        if (ptr != g.size() - 1)
            throw new RuntimeException("Incorrect structure of tree!");
        return edges;
    }

    public int getNumVertexes() {
        return g.size();
    }

    public void addEdge(int from, int to) {
        degree[from]++;
        degree[to]++;
        edges[ptr++] = new Edge(from, to);
        g.get(from).add(new Pair<Integer, Edge>(to, edges[ptr - 1]));
        g.get(to).add(new Pair<Integer, Edge>(from, edges[ptr - 1]));
    }

    public void addEdge(int from, int to, int l, int t) {
        degree[from]++;
        degree[to]++;
        edges[ptr++] = new Edge(from, to, l, t);
        g.get(from).add(new Pair<Integer, Edge>(to, edges[ptr - 1]));
        g.get(to).add(new Pair<Integer, Edge>(from, edges[ptr - 1]));
    }

    public int getDegree(int v) {
        return degree[v];
    }

    public ArrayList<ArrayList<Pair<Integer, Edge>>> getGraph() {
        return g;
    }
}

class Node {
    char sign;    
    String value;
    public ArrayList<Integer> go = new ArrayList<>();
}

public class TestGen {
    int testCount;
    Random random = new Random(0xDEAD);
    public static int MX_N = 100000;
    public static int MX_L = 10000;
    public static int MX_T = 1000000000;

    public String randomString(int len) {
        StringBuilder result = new StringBuilder();
        int n = random.nextInt(len) + 1;
        result.append(random.nextInt(9) + 1 + '0');
        for (int i = 0; i < n; i++) {
            result.append(random.nextInt(10) + '0');
        }
        return result.toString();
    }
    
    public String getExpression(Node[] nodes, int root, int parent) {
        if (nodes[root].sign == '_') {
            return nodes[root].value;
        }
        StringBuilder result = new StringBuilder();
        for (int i = 0; i < nodes[root].go.size(); i++) {
            int u = nodes[root].go.get(i);
            if (u == parent) {
                continue;
            }
            result.append('(');
            result.append(getExpression(nodes, u, root));
            result.append(')');
            if (i < nodes[root].go.size() - 2 || (i < nodes[root].go.size() - 1 && nodes[root].go.get(nodes[root].go.size() - 1) != parent)) {
                result.append(nodes[root].sign);
            }
        }
        return result.toString();
    }
    
    public boolean isDigit(char c) {
        return '0' <= c && c <= '9';
    }
    
    public String addSpaces(String s) {
        int spaces = MX_N - s.length();
        StringBuilder result = new StringBuilder();
        for (int i = 0; i < s.length(); i++) {
            char c = s.charAt(i);
            if (!isDigit(c)) {
                int x = random.nextInt(5);
                for (int j = 0; j < x; j++) {
                    if (spaces > 0) {
                        result.append(' ');
                        spaces--;
                    }
                }
                result.append(c);
                x = random.nextInt(5);
                for (int j = 0; j < x; j++) {
                    if (spaces > 0) {
                        result.append(' ');
                        spaces--;
                    }
                }
            } else {
                result.append(c);
            }
        }
        return result.toString();
    }

    public void write(Tree tree, int len) throws IOException {
        String path = String.format("../tests/%02d", ++testCount);
        PrintWriter out = new PrintWriter(path);
        int n = tree.getNumVertexes();
        Node[] nodes = new Node[n];
        for (int i = 0; i < n; i++) {
            nodes[i] = new Node();
        }
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < tree.g.get(i).size(); j++) {
                nodes[i].go.add(tree.g.get(i).get(j).first);
            }
            if (tree.g.get(i).size() <= 2 && (i != 0 || tree.g.get(i).size() < 2)) {
                nodes[i].value = randomString(len);
                nodes[i].sign = '_';
            } else {
                nodes[i].sign = (random.nextInt(2) == 0 ? '+' : '*');
            }
        }
        String result = getExpression(nodes, 0, -1);
        //result = addSpaces(result);
        out.println(result);
        ArrayList<Integer> pos = new ArrayList<>();
        for (int i = 0; i < result.length(); i++) {
            if (result.charAt(i) == '+' || result.charAt(i) == '*') {
                pos.add(i + 1);
            }
        }
        Collections.shuffle(pos, random);
        out.println(pos.size());
        for (int i = 0; i < pos.size(); i++) {
            out.println(pos.get(i));
        }
        out.close();
        System.out.println(String.format("Test %d is generated", testCount));
    }
    
    public void write(String result, int[] pos) throws IOException {
        String path = String.format("../tests/%02d", ++testCount);
        PrintWriter out = new PrintWriter(path);
        out.println(result);
        out.println(pos.length);
        for (int i = 0; i < pos.length; i++) {
            out.println(pos[i]);
        }
        out.close();
        System.out.println(String.format("Test %d is generated", testCount));
    }

    void genSamples() throws IOException {
        write("2*3+5", new int[]{2, 4});      
        write("2+3+4", new int[]{2, 4});
        write("(2+3)*4", new int[]{3, 6});
    }

    void genRandomTest(int n, int len) throws IOException {
        write(genBinaryStructure(n), len);
        write(genRandomWithBigDiameter(n), len);
        write(genRandomWithManyListsStructure(n), len);
        write(genLongTreeStructure(n, n / 2), len);
        write(genNBranchesStructure(n, random.nextInt(5) + 2), len);
        write(genBoneStructure(n / 3, n / 3, n / 3), len);
    }

    void genMinTests() throws IOException {
        write("2*3*5*(1+3*(1+5)+3)", new int[]{2, 4, 6, 9, 11, 14, 17});
        write("1+1+1+1+1+1+1+1+1+1*10", new int[]{2, 4, 6, 8, 10, 12, 14, 16, 18, 20});
        write("2*2*2*2*2*2*2*2*2*2*2*2*2*2*2*2*2*2*2*2*2*2", new int[]{2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30});
        write("((1*2+3)*2+(2+5)*(1+2)+1)*(2+3)*(999+1+5+2+239+2)", new int[]{4, 6, 9, 11, 14, 17, 20, 23, 26, 29, 32, 37, 39, 41, 43, 47});
    }

    void genMaxTests(int n) throws IOException {
        StringBuilder expression = new StringBuilder();
        for (int i = 0; i < (n - 3) / 2; i++) {
            expression.append('(');
        }
        expression.append("1+1");
        for (int i = 0; i < (n - 3) / 2; i++) {
            expression.append(')');
        }
        String expr = expression.toString();
        int[] pos = new int[100000];
        pos[0] = expr.indexOf('+') + 1;
        for (int i = 0; i < 100000 - 1; i++) {
            pos[i + 1] = pos[0];
        }
        write(expr, pos);

        expression = new StringBuilder();
        for (int i = 0; i < n - 2; i++) {
            expression.append('9');
        }
        expression.append("+1");
        expr = expression.toString();
        pos = new int[100000];
        pos[0] = expr.indexOf('+') + 1;
        for (int i = 0; i < 100000 - 1; i++) {
            pos[i + 1] = pos[0];
        }
        write(expr, pos);
        
        expression = new StringBuilder();
        for (int i = 0; i < n / 2 - 1; i++) {
            expression.append(random.nextInt(9) + 1 + '0');
            expression.append(random.nextInt(2) == 0 ? '+' : '*');
        }
        expression.append(random.nextInt(9) + 1 + '0');
        expr = expression.toString();
        ArrayList<Integer> poss = new ArrayList<>();
        for (int i = 0; i < expr.length(); i++) {
            if (expr.charAt(i) == '+' || expr.charAt(i) == '*') {
                poss.add(i + 1);
            }
        }
        Integer[] need = new Integer[poss.size()];
        poss.toArray(need);
        pos = new int[need.length];
        for (int i = 0; i < need.length; i++) {
            pos[i] = need[i];
        }
        write(expr, pos);
    }

    public ArrayList<Integer> genRandomShuffle(int n) {
        ArrayList<Integer> p = new ArrayList<Integer>();
        for (int i = 0; i < n; ++i)
            p.add(i);
        Collections.shuffle(p, random);
        return p;
    }

    public Tree coloringTree(Tree tree, ArrayList <Integer> coloring) {
        Tree.Edge edges[] = tree.getEdges();
        int j = 0;
        for (Tree.Edge e : edges) {
            e.from = coloring.get(e.from) + 1;
            e.to = coloring.get(e.to) + 1;
        }
        return tree;
    }

    public Tree genBinaryStructure(int n) {
        Tree tree = new Tree(n);
        int i;
        for (i = 0; 2 * i + 2 < n; ++i) {
            tree.addEdge(i, 2 * i + 1);
            tree.addEdge(i, 2 * i + 2);
        }
        i = 2 * i + 1;
        for (; i < n; ++i)
            tree.addEdge(random.nextInt(i), i);
        return tree;
    }

    public Tree genRandomWithBigDiameter(int n) {
        Tree tree = new Tree(n);
        Dsu dsu = new Dsu(n);
        int edges = 0;
        while (edges < n - 1) {
            int x = random.nextInt(n);
            int y = random.nextInt(n);
            if (dsu.join(x, y)) {
                ++edges;
                tree.addEdge(x, y);
            }
        }
        return tree;
    }

    public Tree genRandomWithManyListsStructure(int n) {
        Tree tree = new Tree(n);
        for (int i = 1; i < n; ++i)
            tree.addEdge(random.nextInt(i), i);
        return tree;
    }

    public Tree genLongTreeStructure(int n, int ridgeLen) {
        Tree tree = new Tree(n);
        for (int i = 0; i + 1 < ridgeLen; ++i)
            tree.addEdge(i, i + 1);
        for (int i = ridgeLen; i < n; ++i)
            tree.addEdge(random.nextInt(i), i);
        return tree;
    }

    public Tree genNBranchesStructure(int n, int branches) {
        if (n - 1 < branches)
            throw new RuntimeException("Too much branches!");
        Tree tree = new Tree(n);
        int len = (n - 1) / branches;
        int v = 1;
        for (int i = 0; i < branches && v < n; ++i) {
            tree.addEdge(0, v++);
            for (int j = 1; j < len && v < n; ++j, ++v)
                tree.addEdge(v - 1, v);
        }
        if (v < n) {
            tree.addEdge(v, len);
            ++v;
            while (v < n)
                tree.addEdge(v - 1, v++);
        }
        return tree;
    }

    public Tree genBoneStructure(int middle, int left, int right) {
        Tree tree = new Tree(middle + left + right);
        for (int i = 0; i + 1 < middle; ++i)
            tree.addEdge(i, i + 1);
        for (int i = middle; i < middle + left; ++i)
            tree.addEdge(0, i);
        for (int i = middle + left; i < middle + left + right; ++i)
            tree.addEdge(middle - 1, i);
        return tree;
    }

    public void run() throws IOException {
        new File("../tests").mkdir();

        genSamples();
        genMinTests();
        genRandomTest(10, 5);
        genRandomTest(10, 10);
        genRandomTest(10, 50);
        genRandomTest(100, 5);
        genRandomTest(100, 10);
        genRandomTest(100, 50);
        genRandomTest(1000, 5);
        genRandomTest(1000, 10);
        genRandomTest(1000, 50);
        genRandomTest(10000, 5);
        genRandomTest(10000, 10);
        genRandomTest(10000, 1);
        genRandomTest(10000, 2);
        genMaxTests(100000);
    }
    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
