import java.io.*;
import java.util.*;
import java.util.ArrayList;
import java.util.function.Supplier;
import java.math.*;

public class TestGen {
    int testCount;
    PrintWriter testsDesc;
    static Random random = new Random(787788);
    static final int MAX_C = 1_000_000_000;
    static final int MAX_T = 2000;

    int[][] samples = {
        {11, 13}
    };

    void write(int[] a) throws IOException {
        String path = String.format("../tests/%02d", ++testCount);
        PrintWriter out = new PrintWriter(path);
        out.println(a.length);
        for (Integer x : a) {
            out.println(x);
        }
        out.close();
        System.out.println(String.format("Test %d is generated", testCount));
    }


    void genHand(int[][] a) throws IOException {
        for (int i = 0; i < a.length; i++) {
            write(a[i]);
        }
    }

    boolean isPrime(int x) {
        for (int i = 2; i * i <= x; i++) {
            if (x % i == 0) {
                return false;
            }
        }
        return true;
    }

    void genFirstPrimes() throws IOException {
        int n = MAX_T;
        int a[] = new int[n];
        int cnt = 0, cur = 2;
        while (cnt < n) {
            while (!isPrime(cur)) {
                cur++;
            }
            a[cnt++] = cur++;
        }
        write(a);
    }

    void genBigPrimes() throws IOException {
        int n = MAX_T;
        int a[] = new int[n];
        int start = 999_900_000;
        int cnt = 0;
        while (cnt < n) {                         
            int value = (int)BigInteger.valueOf(start).nextProbablePrime().longValue();
            a[cnt++] = value;
            start = value + 1;
        }
        write(a);
    }

    void genRandPrimes() throws IOException {
        int n = MAX_T;
        int a[] = new int[n];
        int cnt = 0;
        while (cnt < n) {                         
            a[cnt++] = (int)BigInteger.valueOf(random.nextInt(MAX_C - 10_000)).nextProbablePrime().longValue();
        }
        write(a);
    }

    int getDividers(int x) {
        int res = 0;
        for (int i = 1; i * i <= x; i++) {
            if (x % i != 0) {
                continue;
            }
            res++;
            if (i * i != 0) {
                res++;
            }
        }
        return res;
    }

    @SuppressWarnings("unchecked")
    void genSomeBigDividersPrime() throws IOException {
        int n = MAX_T;
        int cnt = 100_000;
        Map.Entry<Integer, Integer> a[] = new Map.Entry[cnt];
        System.out.println("Generating will take some time");
        for (int i = 0; i < cnt; i++) {
            if (i % (cnt / 10) == 0) {
                System.out.println(i / (cnt / 100) + "%");
            }
            int value = (int)BigInteger.valueOf(MAX_C / 10 + random.nextInt(MAX_C - MAX_C / 10)).nextProbablePrime().longValue();
            a[i] = new AbstractMap.SimpleEntry<Integer, Integer>(value, getDividers(value));
        }
        System.out.println("100%");        
        Arrays.sort(a, new Comparator<Map.Entry<Integer, Integer>>() {
            public int compare(Map.Entry<Integer, Integer> a, Map.Entry<Integer, Integer> b) {
                return -Integer.compare(a.getValue(), b.getValue());
            }
        });
        int res[] = new int[n];
        for (int i = 0; i < n; i++) {
            res[i] = a[i].getKey();
        }
        write(res);
    }

    void run() throws IOException {
        new File("../tests").mkdir();

        genHand(samples);
        genFirstPrimes();
        genBigPrimes();
        for (int i = 0; i < 6; i++) {
           genRandPrimes();
        }
        genSomeBigDividersPrime();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}