import java.io.*;
import java.util.*;

public class TestGen {

    public static void main(String[] args) throws Exception {
        new File("../tests").mkdir();

        done = 0;
        genSamples();

        genRand(5, 10, 10);
        genRand(2, 2, 2);
        genRandBigX(5, 10, 10);
        genRandBigX(2, 2, 2);
        genRandBigX23(5, 10, 10);
        genRandBigX23(2, 2, 2);
        genWihtoutPairs(5, 5);
        genSpecials1(5, 10, 10);
        genSpecial2(7, 15, 20);
        genSpecial3(3, 20, 30);
        genSpecial4(5, 50, 15);

        for (int i = 0; i < 10; i++) {
            int n, c;
            c = rand(MAXT / 2) + MAXT / 2 + 1;
            n = rand(c);
            genRand(n, c, MAXN);
            c = rand(MAXT / 2) + MAXT / 2 + 1;
            n = rand(c);
            genRandBigX(n, c, MAXN);
            c = rand(MAXT / 2) + MAXT / 2 + 1;
            n = rand(c);
            genRandBigX23(n, c, MAXN);
            c = rand(MAXT / 2) + MAXT / 2 + 1;
            n = rand(c);
            genRandBigX23(n, c, MAXN);
        }

        genRand(MAXT, MAXT, MAXN);
        genRand(MAXT, MAXT, MAXN);
        genRandBigX(MAXT, MAXT, MAXN);
        genRandBigX(MAXT, MAXT, MAXN);
        genWihtoutPairs(MAXT, MAXN);
        genSpecials1(rand(MAXT), MAXT, MAXN);
        genSpecial2(MAXT, MAXT, MAXN);
        genSpecial3(MAXT, MAXT, MAXN);
        genSpecial4(MAXT, MAXT, MAXN);

    }

    static class Pair {
        int t, a;
        Pair(){}
        Pair(int x, int y) {
            this.t = x;
            this.a = y;
        }
    }

    static int done = 0;
    static final Random rnd = new Random(3289214075201L);
    static final int MAXN = 1000;
    static final int MAXT = 10_000;

    static int rand(int n) {
        if (n < 1)
            return 0;
        return rnd.nextInt(n);
    }

    static int nand(int n) {
        return rand(n / 2) + n / 2;
    }

    static boolean trueOrFalse() {
        return rnd.nextBoolean();
    }

    static void print(int x, int c, Pair[] a) throws IOException {
        done++;
        System.err.println("Generation test #" + done);
        PrintWriter out;
        if (done < 10)
            out = new PrintWriter("../tests/0" + String.valueOf(done));
        else
            out = new PrintWriter("../tests/" + String.valueOf(done));
        out.println(x + " " + a.length + " " + c);
        for (int i = 0; i < a.length; i++) {
            out.println(a[i].t + " " + a[i].a);
        }
        out.close();
    }

    //begin generators

    static void genSamples() throws IOException {
        Pair[] a = new Pair[2];
        a[0] = new Pair(2, 1);
        a[1] = new Pair(3, 1);
        print(1, 5, a);
        a[1] = new Pair(3, 10);
        print(3, 5, a);
    }

    static void genRand(int n, int c, int maxp) throws Exception {
        int x = rand(maxp + 1);
        Pair[] a = new Pair[n];
        int[] times = genTimes(n, c);
        for (int i = 0; i < n; i++) {
            a[i] = new Pair(times[i], rand(maxp) + 1);
        }
        print(x, c, a);
    }

    static void genRandBigX(int n, int c, int maxp) throws Exception {
        int x = rand(maxp / 2) + maxp / 2 + 1;
        Pair[] a = new Pair[n];
        int[] times = genTimes(n, c);
        for (int i = 0; i < n; i++) {
            a[i] = new Pair(times[i], rand(maxp) + 1);
        }
        print(x, c, a);
    }

    static void genRandBigX23(int n, int c, int maxp) throws Exception {
        int x = rand(maxp / 2) + maxp / 2 + 1;
        Pair[] a = new Pair[n];
        int[] times = genTimes(n, n);
        for (int i = 0; i < n; i++) {
            a[i] = new Pair(c - n + times[i], rand(maxp) + 1);
        }
        print(x, c, a);
    }

    static void genWihtoutPairs(int c, int maxp) throws Exception {
        int x = rand(maxp / 2) + maxp / 2 + 1;
        Pair[] a = new Pair[0];
        print(x, c, a);
        print(Math.min(c, maxp), c, a);
        print(Math.min(c / 2, maxp), c, a);
        print(Math.min(c - 1, maxp), c, a);
    }

    static void genSpecials1(int q, int c, int maxp) throws Exception {
        int x = rand(maxp / 2) + maxp / 2 + 1;
        Pair[] a = new Pair[q];
        for (int i = 0; i < q; i++) {
            a[i] = new Pair(c - q + 1 + i, 1);
        }
        print(x, c, a);
        print(Math.min(c, maxp), c, a);
        print(Math.min(c / 2, maxp), c, a);
        print(Math.min(c - 1, maxp), c, a);
    }

    static void genSpecial2(int n, int c, int maxp) throws Exception {
        int x = rand(maxp / 2) + maxp / 2 + 1;
        Pair[] a = new Pair[n];
        int[] times = genTimes(n, c);
        for (int i = 0; i < n; i++) {
            a[i] = new Pair(times[i], rand(maxp) + 1);
        }
        print(x, c, a);
    }

    static void genSpecial3(int n, int c, int maxp) throws Exception {
        int x = rand(maxp / 2) + maxp / 2 + 1;
        Pair[] a = new Pair[n];
        int[] times = genTimes(n, c);
        for (int i = 0; i < n; i++) {
            a[i] = new Pair(times[i], 1);
        }
        print(x, c, a);
    }

    static void genSpecial4(int n, int c, int maxp) throws Exception {
        int x = rand(maxp / 2) + maxp / 2 + 1;
        Pair[] a = new Pair[n];
        int[] times = genTimes(n, c);
        for (int i = 0; i < n; i++) {
            a[i] = new Pair(times[i], maxp);
        }
        print(x, c, a);
    }



    static int[] genTimes(int n, int max) throws Exception {
        assert n <= max;
        int[] time = new int[max];
        for (int i = 0; i < max; i++) {
            if (i < n)
                time[i] = 1;
            else
                time[i] = 0;
        }
        shuffle(time);
        int[] res = new int[n];
        int j = 0;
        for (int i = 0; i < n; i++, j++) {
            while (time[j] != 1)
                j++;
            res[i] = j + 1;
        }
        return res;
    }

    static void shuffle(int[] a) {
        for (int i = 0; i < a.length; i++) {
            int j = rnd.nextInt(i + 1);
            int tmp = a[j];
            a[j] = a[i];
            a[i] = tmp;
        }
    }


    //end generator

}
