/**
 * Created with IntelliJ IDEA.
 * User: User
 * Date: 20.11.14
 * Time: 1:40
 * To change this template use File | Settings | File Templates.
 */

import java.io.*;
import java.util.*;

class Pair<F, S> {
    public F first;
    public S second;
    public Pair(F f, S s) {
        this.first = f;
        this.second = s;
    }
}

class Dsu {
    private int[] p;
    private int[] rank;

    public Dsu(int n) {
        p = new int[n];
        rank = new int[n];
        for (int i = 0; i < n; ++i) p[i] = i;
    }

    public int root(int v) {
        if (v == p[v])
            return v;
        p[v] = root(p[v]);
        return p[v];
    }

    public boolean join(int x, int y) {
        x = root(x);
        y = root(y);
        if (x == y)
            return false;
        if (rank[x] < rank[y]) {
            int tmp = x;
            x = y;
            y = tmp;
        }
        p[y] = x;
        if (rank[x] == rank[y]) ++rank[x];
        return true;
    }
}

class Tree {
    private ArrayList<ArrayList<Integer> > g;
    private int[] value;
    private Edge[] edges;
    private int ptr = 0;

    static class Edge {
        public int from;
        public int to;

        public Edge(int from, int to) {
            this.from = from;
            this.to = to;
        }
    }

    public Tree(int n) {
        value = new int[n];
        edges = new Edge[n - 1];
        g = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            g.add(new ArrayList<Integer>());
    }

    public Edge[] getEdges() {
        if (ptr != g.size() - 1)
            throw new RuntimeException("Incorrect structure of tree!");
        return edges;
    }

    public int getNumVertexes() {
        return g.size();
    }

    public void addEdge(int from, int to) {
        edges[ptr++] = new Edge(from, to);
        g.get(from).add(to);
        g.get(to).add(from);
    }

    public ArrayList<Integer> getAdjacent(int v) {
        return g.get(v);
    }

    public void setValue(int v, int x) {
        value[v] = x;
    }

    public void setValues(int[] a) {
        value = a;
    }

    public int getValue(int v) {
        return value[v];
    }
}

public class TestGen {
    final int MAX_N = 100000;
    final int MAX_VAL = 1000000000;
    Random random = new Random(0xFEED);
    boolean isSamples = false;

    int testCount;

    void print(Tree tree) throws IOException {
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", ++testCount));
        System.out.println(testCount);
        Tree.Edge[] ed = tree.getEdges();
        if (ed.length != tree.getNumVertexes() - 1)
            throw new RuntimeException("Wrong number of edges!");
        ArrayList<Tree.Edge> edges = new ArrayList<>();
        if (!isSamples)
            tree = coloringTree(tree);

        Collections.addAll(edges, ed);
        if (!isSamples) {
            for (Tree.Edge e : edges)
                if (random.nextBoolean()) {
                    int tmp = e.from;
                    e.from = e.to;
                    e.to = tmp;
                }
        }
        out.println(tree.getNumVertexes());
        for (int i = 0; i < tree.getNumVertexes(); ++i)
            if (i + 1 == tree.getNumVertexes()) out.println(tree.getValue(i));
            else out.print(tree.getValue(i) + " ");

        for (int i = 0; i < edges.size(); ++i)
            if (i + 1 == edges.size())
                out.println(edges.get(i).from + " " + edges.get(i).to);
            else
                out.println(edges.get(i).from + " " + edges.get(i).to);
        out.close();
    }
    public void printSample(String s)throws IOException {
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", ++testCount));
        System.out.println(testCount);
        out.println(s);
        out.close();
    }

    public Tree coloringTree(Tree tree) {
        int n = tree.getNumVertexes();
        List<Pair<Integer, Integer>> perm = new ArrayList<>();
        for (int i = 1; i < n; ++i)
            perm.add(new Pair<>(i, tree.getValue(i)));
        //Collections.shuffle(perm, random);
        int[] inv = new int[n];
        for (int i = 0; i < perm.size(); ++i)
            inv[perm.get(i).first] = i + 1;
        Tree.Edge edges[] = tree.getEdges();
        for (Tree.Edge e : edges) {
            if (e.from == 0) e.from = 1;
            else e.from = inv[e.from] + 1;
            if (e.to == 0) e.to = 1;
            else  e.to = inv[e.to] + 1;
        }
        for (int i = 1; i < n; ++i)
            tree.setValue(inv[i], perm.get(i - 1).second);
        return tree;
    }

    public Tree genBinaryStructure(int n) {
        Tree tree = new Tree(n);
        int i;
        for (i = 0; 2 * i + 2 < n; ++i) {
            tree.addEdge(i, 2 * i + 1);
            tree.addEdge(i, 2 * i + 2);
        }
        i = 2 * i + 1;
        for (; i < n; ++i)
            tree.addEdge(random.nextInt(i), i);
        return tree;
    }

    public Tree genRandomWithBigDiameter(int n) {
        Tree tree = new Tree(n);
        Dsu dsu = new Dsu(n);
        int edges = 0;
        while (edges < n - 1) {
            int x = random.nextInt(n);
            int y = random.nextInt(n);
            if (dsu.join(x, y)) {
                ++edges;
                tree.addEdge(x, y);
            }
        }
        return tree;
    }

    public Tree genRandomWithManyListsStructure(int n) {
        Tree tree = new Tree(n);
        for (int i = 1; i < n; ++i)
            tree.addEdge(random.nextInt(i), i);
        return tree;
    }

    public Tree genLongTreeStructure(int n, int ridgeLen) {
        Tree tree = new Tree(n);
        for (int i = 0; i + 1 < ridgeLen; ++i)
            tree.addEdge(i, i + 1);
        for (int i = ridgeLen; i < n; ++i)
            tree.addEdge(random.nextInt(i), i);
        return tree;
    }

    public Tree genNBranchesStructure(int n, int branches) {
        if (n - 1 < branches)
            throw new RuntimeException("Too much branches!");
        Tree tree = new Tree(n);
        int len = (n - 1) / branches;
        int v = 1;
        for (int i = 0; i < branches && v < n; ++i) {
            tree.addEdge(0, v++);
            for (int j = 1; j < len && v < n; ++j, ++v)
                tree.addEdge(v - 1, v);
        }
        if (v < n) {
            tree.addEdge(v, len);
            ++v;
            while (v < n)
                tree.addEdge(v - 1, v++);
        }
        return tree;
    }

    public Tree genBoneStructure(int middle, int left, int right) {
        Tree tree = new Tree(middle + left + right);
        for (int i = 0; i + 1 < middle; ++i)
            tree.addEdge(i, i + 1);
        for (int i = middle; i < middle + left; ++i)
            tree.addEdge(0, i);
        for (int i = middle + left; i < middle + left + right; ++i)
            tree.addEdge(middle - 1, i);
        return tree;
    }


    public Tree randomElementValues(Tree tree) {
        for (int i = 0; i < tree.getNumVertexes(); ++i)
            tree.setValue(i, random.nextInt(MAX_VAL) + 1);
        return tree;
    }

    public void dfsSetValue(int v, int p, Tree tree, Random nextValue) {
        tree.setValue(v, nextValue.nextInt());
        ArrayList<Integer> vertexes = tree.getAdjacent(v);
        for (Integer to : vertexes)
            if (to != p)
                dfsSetValue(to, v, tree, nextValue);
    }

    public Tree monotonicElementValues(Tree tree, Random nextValue) {
        dfsSetValue(0, -1, tree, nextValue);
        return tree;
    }

    public Tree fromRealTreeValues(Tree tree, int change) {
        int n = tree.getNumVertexes();
        for (int i = 0; i < n; ++i)
            tree.setValue(i, random.nextInt(MAX_VAL) + 1);
        int[] mn = new int[n];
        for (int i = 0; i < n; ++i) mn[i] = MAX_VAL + 1;
        dfsCalcMin(0, -1, tree, mn);
        List<Integer> perm = new ArrayList<>();
        for (int i = 0; i < n; ++i) perm.add(i);
        Collections.shuffle(perm, random);
        for (int i = 0; i < change; ++i)
            mn[perm.get(i)] = random.nextInt(MAX_VAL) + 1;
        for (int i = 0; i < n; ++i)
            tree.setValue(i, mn[i]);
        return tree;
    }

    public void dfsCalcMin(int v, int p, Tree tree, int[] mn) {
        ArrayList<Integer> vertexes = tree.getAdjacent(v);
        for (Integer to : vertexes)
            if (p != to)
                dfsCalcMin(to, v, tree, mn);
        mn[v] = Math.min(mn[v], tree.getValue(v));
    }

    public Tree greedyValues(Tree tree, double probably) {
        dfsGreedy(0, -1, tree, probably, 1, MAX_VAL);
        return tree;
    }


    public void dfsGreedy(int v, int p, Tree tree, double probably, int mx, int mn) {
        if (random.nextDouble() < probably) {
            int len = MAX_VAL / 2 / (int)(tree.getNumVertexes() * probably);
            mx += random.nextInt(len);
            mx = Math.min(mx, MAX_VAL);
            tree.setValue(v, mx);
        } else {
            int rn = random.nextInt(100) + 1;
            mn -= rn;
            tree.setValue(v, mn);
        }
        ArrayList<Integer> vertexes = tree.getAdjacent(v);
        for (Integer to : vertexes)
            if (to != p)
                dfsGreedy(to, v, tree, probably, mx, mn);
    }

    void genSamples() throws IOException {
        isSamples = true;
        printSample("5\n" +
                "7 3 1 2 2\n" +
                "1 2\n" +
                "1 3\n" +
                "3 4\n" +
                "3 5");
        printSample("5\n" +
                "3 1 4 5 2\n" +
                "1 2\n" +
                "2 3\n" +
                "1 4\n" +
                "4 5");
        isSamples = false;
    }

    void genHandleTests() throws IOException {
    }

    void run() throws IOException {
        new File("../tests").mkdir();
        genSamples();
        genHandleTests();

        Random increase = new Random() {
            private int x = 0;
            @Override
            public int nextInt() {
                return ++x;
            }
        };

        Random decrease = new Random() {
            private int x = MAX_VAL;
            @Override
            public int nextInt() {
                return --x;
            }
        };

        int n = 1000;
        print(randomElementValues(genLongTreeStructure(n, n)));//1
        print(randomElementValues(genLongTreeStructure(n, n / 2)));//2
        print(randomElementValues(genBinaryStructure(n)));//3
        print(randomElementValues(genBinaryStructure(n)));//4

        print(randomElementValues(genNBranchesStructure(n, 100)));//5
        print(randomElementValues(genNBranchesStructure(n, (int) Math.sqrt(n))));//6
        print(randomElementValues(genNBranchesStructure(n, 10)));//7

        print(monotonicElementValues(genLongTreeStructure(n, n), increase));//8
        print(monotonicElementValues(genLongTreeStructure(n, n), decrease));//9
        print(monotonicElementValues(genBinaryStructure(n), increase));//10
        print(monotonicElementValues(genBinaryStructure(n), decrease));//11

        print(greedyValues(genBoneStructure(n / 3, n / 3, n / 3), 0.7));//12
        print(greedyValues(genBinaryStructure(n), 0.8));//13
        print(greedyValues(genLongTreeStructure(n, n - 10), 0.3));//14
        print(greedyValues(genNBranchesStructure(n, 10), 0.2));//15

        print(fromRealTreeValues(genRandomWithManyListsStructure(n), 0));//16
        print(fromRealTreeValues(genRandomWithBigDiameter(n), 10));//17
        print(fromRealTreeValues(genRandomWithBigDiameter(n), n / 2));//18

        n = MAX_N;
        print(randomElementValues(genBinaryStructure(n)));//19
        print(randomElementValues(genBinaryStructure(n)));//20
        print(randomElementValues(genRandomWithBigDiameter(n)));//21
        print(randomElementValues(genBoneStructure(n / 3, 10000, n - n / 3 - 10000)));//22
        print(randomElementValues(genRandomWithManyListsStructure(n)));//23
        print(randomElementValues(genNBranchesStructure(n, 10)));//24
        print(randomElementValues(genNBranchesStructure(n, n / 10)));//25
        print(randomElementValues(genNBranchesStructure(n, (int)Math.sqrt(n))));//26

        print(monotonicElementValues(genLongTreeStructure(n - 1000, 1000), increase));//27
        print(monotonicElementValues(genLongTreeStructure(n - 1000, 1000), decrease));//28
        print(monotonicElementValues(genBinaryStructure(n), increase));//29
        print(monotonicElementValues(genBinaryStructure(n), decrease));//30
        print(monotonicElementValues(genRandomWithManyListsStructure(n), decrease));//31
        print(monotonicElementValues(genRandomWithBigDiameter(n), decrease));//32
        print(monotonicElementValues(genRandomWithBigDiameter(n), increase));//33

        print(greedyValues(genRandomWithBigDiameter(n), 1));//34
        print(greedyValues(genRandomWithBigDiameter(n), 0.7));//35
        print(greedyValues(genRandomWithBigDiameter(n), 0.3));//36
        print(greedyValues(genBinaryStructure(n), 1));//37
        print(greedyValues(genBinaryStructure(n), 0.7));//38
        print(greedyValues(genBinaryStructure(n), 0.5));//39
        print(greedyValues(genBinaryStructure(n), 0.1));//40
        print(greedyValues(genNBranchesStructure(n, 10), 1));//41
        print(greedyValues(genNBranchesStructure(n, 20), 0.8));//42
        print(greedyValues(genNBranchesStructure(n, (int)Math.sqrt(n)), 1));//43
        print(greedyValues(genNBranchesStructure(n, (int)Math.sqrt(n)), 0.5));//44
        print(greedyValues(genBoneStructure(n / 3, n / 3, n / 3), 0.8));//45
        print(greedyValues(genBoneStructure(n / 3, 10000, n - n / 3 - 10000), 0.3));//46
        print(greedyValues(genBoneStructure(n / 3, 20000, 20000), 0.5));//47
        print(greedyValues(genLongTreeStructure(n, n / 2), 0.5));//48
        print(greedyValues(genLongTreeStructure(n, n / 3), 0.8));//49

        print(fromRealTreeValues(genBinaryStructure(n), 0));//50
        print(fromRealTreeValues(genBinaryStructure(n), 100));//51
        print(fromRealTreeValues(genBinaryStructure(n), n / 2));//52
        print(fromRealTreeValues(genBinaryStructure(n), 10000));//53
        print(fromRealTreeValues(genLongTreeStructure(n, n / 2), 10000));//54
        print(fromRealTreeValues(genLongTreeStructure(n, n / 3), n / 2));//55
        print(fromRealTreeValues(genLongTreeStructure(n, n - 10000), 3 * n / 4));//56
        print(fromRealTreeValues(genLongTreeStructure(n, n / 2), 3 * n / 4));//57
        print(fromRealTreeValues(genNBranchesStructure(n, 10), n / 10));//58
        print(fromRealTreeValues(genNBranchesStructure(n, (int)Math.sqrt(n)), n / 2));//59
        print(fromRealTreeValues(genNBranchesStructure(n, 1000), n / 3));//60
        print(fromRealTreeValues(genNBranchesStructure(n, 100), n / 3));//61
        print(randomElementValues(genNBranchesStructure(n, n - 1)));//62
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}