#include <cstdio>
#include <vector>
#include <algorithm>
#include <iostream>
#include <ctime>
using namespace std;
const int N = 1e5 + 10;

struct Node {
    static Node memory[N];
    static int ptr;

    int x, val, y, lab, sz;
    Node *l, *r;
    Node() {}
    Node(int x, int val):x(x), val(val) {
        y = rand() | (rand()<<15);
        l = r = NULL;
        lab = 0;
        sz = 1;
    }

    void* operator new(size_t size) {
        ptr++;
        return &memory[ptr - 1];
    }
};
int Node::ptr = 0;
Node Node::memory[N];

typedef Node* PNode;

int size(PNode v) {
    if (v == NULL)
        return 0;
    return v->sz;
}

void push(PNode v) {
    if (v) {
        if (v->l) v->l->lab += v->lab;
        if (v->r) v->r->lab += v->lab;
        v->val += v->lab;
        v->lab = 0;
    }
}

PNode update(PNode v) {
    if (v) {
        v->sz = size(v->l) + size(v->r) + 1;
        return v;
    }
    return NULL;
}

PNode merge(PNode L, PNode R) {
    push(L);
    push(R);
    if (!L) return update(R);
    if (!R) return update(L);
    if (L->y > R->y) {
        L->r = merge(L->r, R);
        return update(L);
    } else {
        R->l = merge(L, R->l);
        return update(R);
    }
}

void split(PNode t, int k, PNode& L, PNode& R) {
    push(t);
    if (!t) L = R = NULL;
    else if (t->x < k) {//TODO change ?
        split(t->r, k, t->r, R);
        L = t;
    } else {
        split(t->l, k, L, t->l);
        R = t;
    }
    update(t);
}

void splitByVal(PNode t, int k, PNode& L, PNode& R) {
    push(t);
    if (!t) L = R = NULL;
    else if (t->val > k) {//TODO change ?
        splitByVal(t->r, k, t->r, R);
        L = t;
    } else {
        splitByVal(t->l, k, L, t->l);
        R = t;
    }
    update(t);
}


vector <int> g[N];
int a[N];
void getSubtree(PNode v, vector <PNode>& dp) {
    if (v == NULL)
        return;
    push(v);
    getSubtree(v->l, dp);
    dp.push_back(v);
    getSubtree(v->r, dp);
}
PNode first(PNode v) {
    if (v == NULL)
        return NULL;
    push(v);
    while (v->l) {
        v = v->l;
        push(v);
    }
    return v;
}

PNode unionTrees(PNode v1, PNode v2) {
    if (size(v1) > size(v2)) swap(v1, v2);
    vector <PNode> dp;
    getSubtree(v1, dp);
    for (int i = 0; i < dp.size(); ++i)
        dp[i]->l = dp[i]->r = NULL;
    PNode ret = NULL;
    PNode L;
    int mx = 0;
    for (int i = 0; i < dp.size(); ++i) {
        mx = max(mx, dp[i]->val);
        split(v2, dp[i]->x, L, v2);
        if (L != NULL) L->lab += mx;
        PNode f = first(v2);
        int add = (f == NULL ? 0 : f->val);
        dp[i]->val = mx + add;
        ret = merge(merge(ret, L), dp[i]);
    }
    return merge(ret, v2);
}

PNode dfs(int v, int parent) {
    PNode ret = NULL;
    for (int i = 0; i < g[v].size(); ++i) {
        if (g[v][i] == parent) continue;
        PNode son = dfs(g[v][i], v);
        ret = unionTrees(son, ret);
    }
    //if (v == 1) printf("f0 = %d\n", first(ret)->val);
    PNode L, M, R;
    split(ret, a[v], M, R);
    int add = (R == NULL ? 0 : first(R)->val);
    PNode ins = new Node(a[v], 1 + add);
    splitByVal(M, add, L, M);
    //if (M) M->lab++;
    ret = merge(merge(merge(L, M), ins), R);
    return ret;
}

int main() {
    int startTime = clock();
    srand(time(NULL));
    freopen("treemin.in", "r", stdin);
    freopen("treemin.out", "w", stdout);
    int n;
    scanf("%d", &n);
    for (int i = 0; i < n; ++i) scanf("%d", &a[i]);
    for (int i = 0; i < n - 1; ++i) {
        int x, y;
        scanf("%d%d", &x, &y);
        --x;
        --y;
        g[x].push_back(y);
        g[y].push_back(x);
    }
    PNode v = dfs(0, -1);
    printf("%d\n", n - first(v)->val);
    fprintf(stderr, "%.3f\n", (clock() - startTime) * 1.0 / CLOCKS_PER_SEC);

    /*
10
3 7 5 3 1 2 2 9 1 3
1 2
1 3
2 4
2 5
5 6
6 7
3 8
3 9
3 10
7

4
5 9 1 3
1 2
1 3
1 4
3

5
7 3 1 2 2
1 2
1 3
3 4
3 5
4

4
7 1 2 2
1 2
2 3
2 4
3

10
1 7 5 3 1 2 2 9 1 3
1 2
1 3
2 4
2 5
5 6
6 7
3 8
3 9
3 10
8
    */
    return 0;
}
